@@ -10,7 +10,7 @@ from requests_futures.sessions import FuturesSession
 
 from arize import public_pb2 as public__pb2
 from arize.bounded_executor import BoundedExecutor
-from arize.utils.types import ModelTypes
+from arize.utils.types import ModelTypes, Embedding
 from arize.model import (
     TrainingRecords,
     ValidationRecords,
@@ -26,6 +26,8 @@ from arize.utils.utils import (
     get_bulk_records,
 )
 
+from warnings import warn
+
 from arize.__init__ import __version__
 
 
@@ -218,7 +220,7 @@ class Client:
     def __init__(
         self,
         api_key: str,
-        organization_key: str,
+        space_key: str,
         uri="https://api.arize.com/v1",
         max_workers=8,
         max_queue_bound=5000,
@@ -227,14 +229,14 @@ class Client:
     ):
         """
         :params api_key: (str) api key associated with your account with Arize AI
-        :params organization_key: (str) organization key in Arize AI
-        :params max_workers: (int) number of max concurrent requests to Arize. Default: 20
+        :params space_key: (str) space key in Arize AI
+        :params max_workers: (int) number of max concurrent requests to Arize. Default: 8
         :max_queue_bound: (int) number of maximum concurrent future objects being generated for publishing to Arize. Default: 5000
         """
 
-        if not isinstance(organization_key, str):
+        if not isinstance(space_key, str):
             raise TypeError(
-                f"organization_key {organization_key} is type {type(organization_key)}, but must be a str"
+                f"space_key {space_key} is type {type(space_key)}, but must be a str"
             )
         self._retry_attempts = retry_attempts
         self._uri = uri + "/log"
@@ -242,7 +244,7 @@ class Client:
         self._stream_uri = uri + "/preprod"
         self._files_uri = uri + "/files"
         self._api_key = api_key
-        self._organization_key = organization_key
+        self._space_key = space_key
         self._timeout = timeout
         self._session = FuturesSession(
             executor=BoundedExecutor(max_queue_bound, max_workers)
@@ -250,7 +252,7 @@ class Client:
         # Grpc-Metadata prefix is required to pass non-standard md through via grpc-gateway
         self._header = {
             "authorization": api_key,
-            "Grpc-Metadata-organization": organization_key,
+            "Grpc-Metadata-space": space_key,
             "Grpc-Metadata-sdk-version": __version__,
             "Grpc-Metadata-sdk": "py",
         }
@@ -271,6 +273,7 @@ class Client:
         ] = None,
         model_type: Optional[ModelTypes] = None,
         prediction_timestamp: Optional[int] = None,
+        embedding_features: Optional[Dict[Union[str, int, float], Embedding]] = None,
     ) -> cf.Future:
         """Logs a record to Arize via a POST request. Returns :class:`Future` object.
         :param model_id: (str) Unique identifier for a given model
@@ -279,10 +282,11 @@ class Client:
         :param prediction_label: (one of str, bool, int, float, Tuple[str, float]) The predicted value for a given model input.
         :param actual_label: (one of str, bool, int, float) The actual true value for a given model input. This actual will be matched to the prediction with the same prediction_id as the one in this call.
         :param shap_values: (str, float) Dictionary containing human readable and debuggable model features keys, along with SHAP feature importance values. Keys must be str, while values must be float.
-        :param features: ((str, int, float), <value>) Optional dictionary containing human readable and debuggable model features. Keys must be str, values one of str, bool, float, long.
+        :param features: ((str, int, float), <value>) Optional dictionary containing human readable and debuggable model features. Keys must be one of str, int, or float. Values must be one of str, bool, float, long.
         :param tags: ((str, int, float), <value>) Optional dictionary containing human readable and debuggable model tags. Keys must be str, values one of str, bool, float, long.
         :param model_type: (ModelTypes) Declares what model type this prediction is for. Binary, Numeric, Categorical, Score_Categorical.
         :param prediction_timestamp: (int) Optional field with unix epoch time in seconds to overwrite timestamp for prediction. If None, prediction uses current timestamp.
+        :param embedding_features ((str, int, float), Embedding): Optional dictionary containing model embedding features. Keys must be one of str, int, or float. Values must be of Embedding type.
         :rtype : concurrent.futures.Future
         """
 
@@ -293,21 +297,38 @@ class Client:
             )
 
         # Validate feature types
-        if features is not None and bool(features):
+        if features:
             for k, v in features.items():
                 val = convert_element(v)
                 if val is not None and not isinstance(val, (str, bool, float, int)):
                     raise TypeError(
                         f"feature {k} with value {v} is type {type(v)}, but expected one of: str, bool, float, int"
                     )
+                if isinstance(k, str) and k.endswith("_shap"):
+                    raise ValueError(
+                        f"feature {k} must not be named with a `_shap` suffix"
+                    )
+
+        # Validate embedding_features type
+        if embedding_features:
+            for emb_name, emb_obj in embedding_features.items():
+                # Must verify embedding type
+                if type(emb_obj) != Embedding:
+                    raise TypeError(
+                        f'Embedding feature "{emb_name}" must be of embedding type'
+                    )
+                Embedding.validate_embedding_object(emb_name, emb_obj)
+
         # Validate tag types
-        if tags is not None and bool(tags):
+        if tags:
             for k, v in tags.items():
                 val = convert_element(v)
                 if val is not None and not isinstance(val, (str, bool, float, int)):
                     raise TypeError(
                         f"tag {k} with value {v} is type {type(v)}, but expected one of: str, bool, float, int"
                     )
+                if isinstance(k, str) and k.endswith("_shap"):
+                    raise ValueError(f"tag {k} must not be named with a `_shap` suffix")
 
         # Check the timestamp present on the event
         if prediction_timestamp is not None and not isinstance(
@@ -352,6 +373,18 @@ class Client:
                         converted_feats[str(k)] = val
                 feats = public__pb2.Prediction(features=converted_feats)
                 p.MergeFrom(feats)
+
+            if embedding_features is not None:
+                converted_embedding_feats = {}
+                for (k, v) in embedding_features.items():
+                    val = get_value_object(value=v, name=k)
+                    if val is not None:
+                        converted_embedding_feats[str(k)] = val
+                embedding_feats = public__pb2.Prediction(
+                    features=converted_embedding_feats
+                )
+                p.MergeFrom(embedding_feats)
+
             if tags is not None:
                 converted_tags = {}
                 for (k, v) in tags.items():
@@ -360,6 +393,7 @@ class Client:
                         converted_tags[str(k)] = val
                 tgs = public__pb2.Prediction(tags=converted_tags)
                 p.MergeFrom(tgs)
+
             if prediction_timestamp is not None:
                 p.timestamp.MergeFrom(get_timestamp(prediction_timestamp))
 
@@ -375,6 +409,15 @@ class Client:
                     value=actual_label, name="actual", model_type=model_type
                 )
             )
+            # Added to support latent tags on actuals.
+            if tags is not None:
+                converted_tags = {}
+                for (k, v) in tags.items():
+                    val = get_value_object(value=v, name=k)
+                    if val is not None:
+                        converted_tags[str(k)] = val
+                tgs = public__pb2.Actual(tags=converted_tags)
+                a.MergeFrom(tgs)
 
         # Validate and construct the optional feature importances
         fi = None
@@ -384,6 +427,10 @@ class Client:
                     raise TypeError(
                         f"feature {k} with value {v} is type {type(v)}, but expected one of: float"
                     )
+                if isinstance(k, str) and k.endswith("_shap"):
+                    raise ValueError(
+                        f"feature {k} must not be named with a `_shap` suffix"
+                    )
             fi = public__pb2.FeatureImportances(feature_importances=shap_values)
 
         if p is None and a is None and fi is None:
@@ -392,7 +439,7 @@ class Client:
             )
 
         rec = public__pb2.Record(
-            organization_key=self._organization_key,
+            space_key=self._space_key,
             model_id=model_id,
             prediction_id=str(prediction_id),
             prediction=p,
@@ -401,6 +448,7 @@ class Client:
         )
         return self._post(record=rec, uri=self._uri, indexes=None)
 
+    # Deprecated
     def bulk_log(
         self,
         model_id: str,
@@ -433,6 +481,11 @@ class Client:
         :param prediction_timestamps: (list<int>) Optional list with same number of elements as prediction_labels field with unix epoch time in seconds to overwrite timestamp for each prediction. If None, prediction uses current timestamp.
         :rtype : list<concurrent.futures.Future>
         """
+        warn(
+            "The method bulk_log is deprecated and will be removed in a future version",
+            DeprecationWarning,
+            stacklevel=2,
+        )
 
         # Validate model_id
         if not isinstance(model_id, str):
@@ -499,6 +552,10 @@ class Client:
                     raise TypeError(
                         f"shap_values.column {name} is type {type(name)}, but expect str"
                     )
+                if isinstance(name, str) and name.endswith("_shap"):
+                    raise ValueError(
+                        f"feature {name} must not be named with a `_shap` suffix"
+                    )
 
         prediction_ids = prediction_ids.to_numpy()
         prediction_labels = (
@@ -511,9 +568,19 @@ class Client:
         )
         if features is not None:
             feature_names = feature_names_overwrite or features.columns
+            for name in feature_names:
+                if isinstance(name, str) and name.endswith("_shap"):
+                    raise ValueError(
+                        f"feature {name} must not be named with a `_shap` suffix"
+                    )
             features = features.to_numpy()
         if tags is not None:
             tag_names = tag_names_overwrite or tags.columns
+            for name in tag_names:
+                if isinstance(name, str) and name.endswith("_shap"):
+                    raise ValueError(
+                        f"tag {name} must not be named with a `_shap` suffix"
+                    )
             tags = tags.to_numpy()
         actual_labels = actual_labels.to_numpy() if actual_labels is not None else None
         shap_columns = shap_values.columns if shap_values is not None else None
@@ -588,10 +655,11 @@ class Client:
             )
             records.append(rec)
         brs = get_bulk_records(
-            self._organization_key, model_id, model_version, bundle_records(records)
+            self._space_key, model_id, model_version, bundle_records(records)
         )
         return self._post_bulk(records=brs, uri=self._bulk_url)
 
+    # Deprecated
     def log_validation_records(
         self,
         model_id: str,
@@ -620,8 +688,13 @@ class Client:
         :param prediction_timestamps: (list<int>) Optional list with same number of elements as prediction_labels field with unix epoch time in seconds to overwrite timestamp for each prediction. If None, prediction uses current timestamp.
         :rtype : list<concurrent.futures.Future>
         """
+        warn(
+            "The method log_validation_records is deprecated and will be removed in a future version",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         rec = ValidationRecords(
-            organization_key=self._organization_key,
+            space_key=self._space_key,
             model_id=model_id,
             model_type=model_type,
             model_version=model_version,
@@ -637,6 +710,7 @@ class Client:
         rec.validate_inputs()
         return self._post_preprod(records=rec.build_proto())
 
+    # Deprecated
     def log_training_records(
         self,
         model_id: str,
@@ -659,8 +733,13 @@ class Client:
         :param tags: Optional 2-D Pandas DataFrame containing human readable and debuggable model tags. DataFrames columns (df.columns) should contain tag names and must have same number of rows as actual_labels and prediction_labels. N.B. np.nan values are stripped from the record and manifest on our platform as a missing value (not 0.0 or NaN)
         :rtype : list<concurrent.futures.Future>
         """
+        warn(
+            "The method log_training_records is deprecated and will be removed in a future version",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         rec = TrainingRecords(
-            organization_key=self._organization_key,
+            space_key=self._space_key,
             model_id=model_id,
             model_type=model_type,
             model_version=model_version,
@@ -698,6 +777,11 @@ class Client:
         :param time_overwrite: (int) Optional field with unix epoch time in seconds to overwrite timestamp for prediction. If None, prediction uses current timestamp.
         :rtype : concurrent.futures.Future
         """
+        warn(
+            "The method log_prediction is deprecated and will be removed in a future version",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self.log(
             model_id=model_id,
             prediction_id=prediction_id,
@@ -725,6 +809,11 @@ class Client:
         :param model_type: (ModelTypes) Declares what model type this prediction is for. Binary, Numeric, Categorical, Score_Categorical.
         :rtype : concurrent.futures.Future
         """
+        warn(
+            "The method log_actual is deprecated and will be removed in a future version",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self.log(
             model_id=model_id,
             model_type=model_type,
@@ -745,6 +834,11 @@ class Client:
         :param shap_values: (str, float) Dictionary containing human readable and debuggable model features keys, along with SHAP feature importance values. Keys must be str, while values must be float.
         :rtype : concurrent.futures.Future
         """
+        warn(
+            "The method log_shap_values is deprecated and will be removed in a future version",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self.log(
             prediction_id=prediction_id,
             model_id=model_id,
@@ -764,6 +858,11 @@ class Client:
         :param shap_values: 1-D Pandas DataFrame or Series. The SHAP value sets for a set of predictions. SHAP value sets are correspond to the prediction ids with the same index.
         :rtype : list<concurrent.futures.Future>
         """
+        warn(
+            "The method log_bulk_shap_values is deprecated and will be removed in a future version",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self.bulk_log(
             model_id=model_id, prediction_ids=prediction_ids, shap_values=shap_values
         )
@@ -797,6 +896,11 @@ class Client:
         :param time_overwrite: (list<int>) Optional list with same number of elements as prediction_labels field with unix epoch time in seconds to overwrite timestamp for each prediction. If None, prediction uses current timestamp.
         :rtype : list<concurrent.futures.Future>
         """
+        warn(
+            "The method log_bulk_predictions is deprecated and will be removed in a future version",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self.bulk_log(
             model_id=model_id,
             model_type=model_type,
@@ -825,6 +929,11 @@ class Client:
         :param actual_labels: 1-D Pandas DataFrame or Series. The actual true values for a given model input. Values are associates to the labels in the same index.
         :rtype : list<concurrent.futures.Future>
         """
+        warn(
+            "The method log_bulk_actuals is deprecated and will be removed in a future version",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return self.bulk_log(
             model_id=model_id,
             model_type=model_type,
