@@ -21,11 +21,11 @@ from arize.utils.utils import (
 class BaseRecord(ABC):
     def __init__(
         self,
-        organization_key: str,
+        space_key: str,
         model_id: str,
         model_type: Optional[ModelTypes] = None,
     ):
-        self.organization_key = organization_key
+        self.space_key = space_key
         self.model_id = model_id
         self.model_type = model_type
 
@@ -38,9 +38,9 @@ class BaseRecord(ABC):
         pass
 
     def _base_validation(self):
-        if not isinstance(self.organization_key, str):
+        if not isinstance(self.space_key, str):
             raise TypeError(
-                f"organization_key {self.organization_key} is type {type(self.organization_key)}, but must be a str"
+                f"space_key {self.space_key} is type {type(self.space_key)}, but must be a str"
             )
         if not isinstance(self.model_id, str):
             raise TypeError(
@@ -95,12 +95,13 @@ class BaseRecord(ABC):
 class PreProductionRecords(BaseRecord, ABC):
     def __init__(
         self,
-        organization_key: str,
+        space_key: str,
         model_id: str,
         model_version: str,
         prediction_labels: Union[pd.DataFrame, pd.Series],
         actual_labels: Union[pd.DataFrame, pd.Series],
         features: Optional[pd.DataFrame] = None,
+        tags: Optional[pd.DataFrame] = None,
         model_type: Optional[ModelTypes] = None,
         prediction_scores: Optional[Union[pd.DataFrame, pd.Series]] = None,
         prediction_ids: Optional[Union[pd.DataFrame, pd.Series]] = None,
@@ -114,11 +115,10 @@ class PreProductionRecords(BaseRecord, ABC):
                 )
             else:
                 model_type = ModelTypes.SCORE_CATEGORICAL
-        super().__init__(
-            organization_key=organization_key, model_id=model_id, model_type=model_type
-        )
+        super().__init__(space_key=space_key, model_id=model_id, model_type=model_type)
         self.model_version = model_version
         self.features = features
+        self.tags = tags
         self.prediction_labels = prediction_labels
         self.prediction_scores = prediction_scores
         self.actual_labels = actual_labels
@@ -161,22 +161,45 @@ class PreProductionRecords(BaseRecord, ABC):
                     f"prediction_labels: {self.prediction_labels.shape[0]}. "
                 )
 
-        if self.features is None:
-            return
-        if not isinstance(self.features, pd.DataFrame):
-            raise TypeError(
-                f"features is type {type(self.features)}, but expect type pd.DataFrame."
-            )
-        if self.features.shape[0] != self.prediction_labels.shape[0]:
-            raise ValueError(
-                f"features has {self.features.shape[0]} sets of features, but must match size of prediction_labels: "
-                f"{self.prediction_labels.shape[0]}. "
-            )
-        for name in self.features.columns:
-            if not isinstance(name, str):
+        if self.features is not None:
+            if not isinstance(self.features, pd.DataFrame):
                 raise TypeError(
-                    f"features.column {name} is type {type(name)}, but expect str"
+                    f"features is type {type(self.features)}, but expect type pd.DataFrame."
                 )
+            if self.features.shape[0] != self.prediction_labels.shape[0]:
+                raise ValueError(
+                    f"features has {self.features.shape[0]} sets of features, but must match size of prediction_labels: "
+                    f"{self.prediction_labels.shape[0]}. "
+                )
+            for name in self.features.columns:
+                if not isinstance(name, str):
+                    raise TypeError(
+                        f"features.column {name} is type {type(name)}, but expect str"
+                    )
+                if isinstance(name, str) and name.endswith("_shap"):
+                    raise ValueError(
+                        f"features.column {name} must not be named with a `_shap` suffix"
+                    )
+
+        if self.tags is not None:
+            if not isinstance(self.tags, pd.DataFrame):
+                raise TypeError(
+                    f"tags is type {type(self.tags)}, but expect type pd.DataFrame."
+                )
+            if self.tags.shape[0] != self.prediction_labels.shape[0]:
+                raise ValueError(
+                    f"tags has {self.tags.shape[0]} sets of tags, but must match size of prediction_labels: "
+                    f"{self.prediction_labels.shape[0]}. "
+                )
+            for name in self.tags.columns:
+                if not isinstance(name, str):
+                    raise TypeError(
+                        f"tags.column {name} is type {type(name)}, but expect str"
+                    )
+                if isinstance(name, str) and name.endswith("_shap"):
+                    raise ValueError(
+                        f"tags.column {name} must not be named with a `_shap` suffix"
+                    )
 
     def _normalize_inputs(self):
         """Converts inputs from DataFrames, Series, lists to numpy arrays or lists for consistent iterations
@@ -192,26 +215,31 @@ class PreProductionRecords(BaseRecord, ABC):
         if isinstance(self.features, pd.DataFrame):
             self.feature_names = self.features.columns
             self.features = self.features.to_numpy()
+        if isinstance(self.tags, pd.DataFrame):
+            self.tag_names = self.tags.columns
+            self.tags = self.tags.to_numpy()
 
 
 class TrainingRecords(PreProductionRecords):
     def __init__(
         self,
-        organization_key: str,
+        space_key: str,
         model_id: str,
         model_version: str,
         prediction_labels: Union[pd.DataFrame, pd.Series],
         actual_labels: Union[pd.DataFrame, pd.Series],
         features: Optional[pd.DataFrame] = None,
+        tags: Optional[pd.DataFrame] = None,
         model_type: Optional[ModelTypes] = None,
         prediction_scores: Optional[Union[pd.DataFrame, pd.Series]] = None,
     ):
         super().__init__(
-            organization_key=organization_key,
+            space_key=space_key,
             model_id=model_id,
             model_type=model_type,
             model_version=model_version,
             features=features,
+            tags=tags,
             prediction_labels=prediction_labels,
             prediction_scores=prediction_scores,
             actual_labels=actual_labels,
@@ -246,12 +274,21 @@ class TrainingRecords(PreProductionRecords):
                 feats = public__pb2.Prediction(features=converted_feats)
                 p.MergeFrom(feats)
 
+            if self.tags is not None:
+                converted_tags = {}
+                for column, name in enumerate(self.tag_names):
+                    val = get_value_object(value=self.tags[row][column], name=name)
+                    if val is not None:
+                        converted_tags[name] = val
+                tgs = public__pb2.Prediction(tags=converted_tags)
+                p.MergeFrom(tgs)
+
             panda = public__pb2.PredictionAndActual(
                 prediction=p,
                 actual=a,
             )
             r = public__pb2.Record(
-                organization_key=self.organization_key,
+                space_key=self.space_key,
                 model_id=self.model_id,
                 prediction_and_actual=panda,
             )
@@ -266,24 +303,26 @@ class TrainingRecords(PreProductionRecords):
 class ValidationRecords(PreProductionRecords):
     def __init__(
         self,
-        organization_key: str,
+        space_key: str,
         model_id: str,
         model_version: str,
         batch_id: str,
         prediction_labels: Union[pd.DataFrame, pd.Series],
         actual_labels: Union[pd.DataFrame, pd.Series],
         features: Optional[pd.DataFrame] = None,
+        tags: Optional[pd.DataFrame] = None,
         model_type: Optional[ModelTypes] = None,
         prediction_scores: Optional[Union[pd.DataFrame, pd.Series]] = None,
         prediction_ids: Optional[Union[pd.DataFrame, pd.Series]] = None,
         prediction_timestamps: Optional[Union[List[int], pd.Series]] = None,
     ):
         super().__init__(
-            organization_key=organization_key,
+            space_key=space_key,
             model_id=model_id,
             model_type=model_type,
             model_version=model_version,
             features=features,
+            tags=tags,
             prediction_labels=prediction_labels,
             prediction_scores=prediction_scores,
             actual_labels=actual_labels,
@@ -343,13 +382,22 @@ class ValidationRecords(PreProductionRecords):
                 feats = public__pb2.Prediction(features=converted_feats)
                 p.MergeFrom(feats)
 
+            if self.tags is not None:
+                converted_tags = {}
+                for column, name in enumerate(self.tag_names):
+                    val = get_value_object(value=self.tags[row][column], name=name)
+                    if val is not None:
+                        converted_tags[name] = val
+                tgs = public__pb2.Prediction(tags=converted_tags)
+                p.MergeFrom(tgs)
+
             panda = public__pb2.PredictionAndActual(
                 prediction=p,
                 actual=a,
             )
             r = public__pb2.Record(
                 prediction_id=prediction_id,
-                organization_key=self.organization_key,
+                space_key=self.space_key,
                 model_id=self.model_id,
                 prediction_and_actual=panda,
             )
